using System;
using System.Collections;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using System.Web.UI;
using System.Globalization;

namespace BeyondBasicUnitTesting.WebUI
{
    public static class WebUiUtility
    {
        public static void AddMessage(Page page, Exception ex)
        {
            if (page != null)
            {
                AddMessage(page, ex.ToString());
            }
        }

        public static void AddMessage(Page page, string message)
        {
            if (page != null)
            {
                page.Validators.Add(new ValidatorMessage(message));
            }
        }

        public static string GetValue(HiddenField control)
        {
            return SafeGetValue(control);
        }

        public static string GetValue(TextBox control)
        {
            return SafeGetValue(control);
        }

        public static string GetValue(Label control)
        {
            return SafeGetValue(control);
        }

        public static bool GetValueAsBool(HiddenField control)
        {
            bool returnValue = false;

            if (Boolean.TryParse(control.Value, out returnValue) == true)
            {
                return returnValue;
            }
            else
            {
                return returnValue;
            }
        }

        public static DateTime GetValueAsDateTime(TextBox control)
        {
            return GetValueAsDateTime(SafeGetValue(control), DateTime.Now);
        }

        public static DateTime GetValueAsDateTime(Label control)
        {
            return GetValueAsDateTime(SafeGetValue(control), DateTime.Now);
        }

        private static DateTime GetValueAsDateTime(string value, DateTime defaultValue)
        {
            DateTime returnValue;

            if (DateTime.TryParse(value, out returnValue) == true)
            {
                return returnValue;
            }
            else
            {
                return defaultValue;
            }
        }

        public static Guid GetValueAsGuid(HiddenField control)
        {
            return GetValueAsGuid(control);
        }

        public static int GetValueAsInt32(TextBox control)
        {
            return GetValueAsInt32(SafeGetValue(control), 0);
        }

        public static int GetValueAsInt32(ListControl control, int defaultReturnValue)
        {
            return GetValueAsInt32(control.SelectedValue, defaultReturnValue);
        }

        public static int GetValueAsInt32(Label control)
        {
            return GetValueAsInt32(SafeGetValue(control), 0);
        }
        
        public static int GetValueAsInt32(HiddenField control)
        {
            return GetValueAsInt32(SafeGetValue(control), 0);
        }

        public static void SetValue(TextBox control, int value)
        {
            SetValue(control, value.ToString());
        }

        public static string SafeGetValue(HiddenField control)
        {
            if (control != null)
            {
                return control.Value;
            }
            else
            {
                return String.Empty;
            }
        }

        public static string SafeGetValue(Label control)
        {
            if (control != null)
            {
                return control.Text;
            }
            else
            {
                return String.Empty;
            }
        }

        public static string SafeGetValue(TextBox control)
        {
            if (control != null)
            {
                return control.Text;
            }
            else
            {
                return String.Empty;
            }
        }

        public static string GetSelectedText(ListControl control, string defaultReturnValue)
        {
            if (control.SelectedItem == null)
            {
                return defaultReturnValue;
            }
            else
            {
                return control.SelectedItem.Text;
            }
        }

        public static void SetValue(TextBox control, string value)
        {
            control.Text = SafeToString(value);
        }

        public static void SetValue(Label control, string value)
        {
            control.Text = SafeToString(value);
        }

        public static void SetValue(Label control, DateTime value)
        {
            control.Text = value.ToString();
        }

        public static void SetValue(Label control, DateTime value, bool dateOnly)
        {
            if (dateOnly == true)
            {
                control.Text = value.ToString("MM/dd/yyyy");
            }
            else
            {
                SetValue(control, value);
            }
        }

        public static void SetValue(Label control, float value)
        {
            control.Text = value.ToString("C").Replace(",", "");
        }

        public static void SetValue(HiddenField control, int value)
        {
            control.Value = SafeToString(value);
        }

        public static void SetValue(Label control, int value)
        {
            control.Text = value.ToString();
        }

        public static void SetValue(ListControl control, int value)
        {
            SelectListItem(control, value.ToString());
        }

        public static void SetValue(HiddenField control, Guid value)
        {
            control.Value = value.ToString();
        }

        public static void SetValue(TextBox control, float value)
        {
            control.Text = value.ToString("C").Replace(",", "");
        }

        public static void SetValue(TextBox control, DateTime value)
        {
            control.Text = value.ToString();
        }

        public static void SetValue(TextBox control, DateTime value, bool dateOnly)
        {
            if (dateOnly == true)
            {
                control.Text = value.ToString("MM/dd/yyyy");
            }
            else
            {
                SetValue(control, value);
            }
        }

        public static void SetValue(HiddenField control, string value)
        {
            control.Value = SafeToString(value);
        }

        public static IList<string> GetAllValuesFromListControl(ListControl control)
        {
            List<string> returnValue = new List<string>();
            if (control != null)
            {
                foreach (ListItem item in control.Items)
                {
                    returnValue.Add(item.Value);
                }
            }

            return returnValue;
        }

        public static void PopulateListControl(ListControl control, IList<string> values)
        {
            if (control == null)
            {
                throw new ArgumentNullException("control", "Argument cannot be null.");
            }

            control.DataSource = null;
            control.Items.Clear();

            foreach (string val in values)
            {
                control.Items.Add(val);
            }
        }

        public static int ParseInt32(string value, int defaultReturnValue)
        {
            int returnValue;

            if (Int32.TryParse(value, out returnValue) == true)
            {
                return returnValue;
            }
            else
            {
                return defaultReturnValue;
            }
        }

        public static int GetValueAsInt32(string value, int defaultReturnValue)
        {
            if (value == null)
            {
                return defaultReturnValue;
            }
            else if (value == "-1")
            {
                return -1;
            }
            else
            {
                int returnValue = 0;

                if (Int32.TryParse(value, out returnValue) == true)
                {
                    return returnValue;
                }
                else
                {
                    return defaultReturnValue;
                }
            }
        }

        public static float GetValueAsFloat(Label control)
        {
            return GetValueAsFloat(control.Text, 0f);
        }

        public static float GetValueAsFloat(TextBox control)
        {
            return GetValueAsFloat(control.Text, 0f);
        }

        public static float GetValueAsFloat(string value, float defaultReturnValue)
        {
            if (value == null)
            {
                return defaultReturnValue;
            }
            else if (value == "-1")
            {
                return -1;
            }
            else
            {
                float returnValue = 0;

                if (Single.TryParse(value, out returnValue) == true)
                {
                    return returnValue;
                }
                else
                {
                    if (Single.TryParse(value, System.Globalization.NumberStyles.Currency, 
                        null,
                        out returnValue) == true)
                    {
                        return returnValue;
                    }
                    else 
                    {
                        return defaultReturnValue;
                    }
                }
            }
        }

        public static string SafeToString(string value, string defaultReturnValue)
        {
            if (value == null)
            {
                return defaultReturnValue;
            }
            else
            {
                return value;
            }
        }

        public static string SafeToString(string value)
        {
            return SafeToString(value, String.Empty);
        }

        public static string SafeToString(float value)
        {
            return SafeToString(value.ToString(), String.Empty);
        }

        private static void UnselectAll(ListControl control)
        {
            if (control != null)
            {
                foreach (ListItem item in control.Items)
                {
                    if (item.Selected == true)
                    {
                        item.Selected = false;
                    }
                }
            }
        }

        public static void SetSelectedValues(ListControl control, IList<string> fieldValues)
        {
            control.ClearSelection();

            if (fieldValues == null || fieldValues.Count == 0)
            {
                // do nothing
            }
            else
            {
                foreach (string val in fieldValues)
                {
                    WebUiUtility.SelectListItem(control, val);
                }
            }
        }

        public static string GetValue(ListControl control)
        {
            if (control == null)
            {
                throw new ArgumentNullException("control", "Argument cannot be null.");
            }
            else
            {
                return SafeToString(control.SelectedValue);
            }
        }

        public static void SelectListItem(ListControl control, string selectedValue)
        {
            SelectListItem(control, selectedValue, selectedValue);
        }

        public static void SelectListItem(ListControl control, string selectedValue, string selectedText)
        {
            bool foundListItem = false;

            foreach (ListItem item in control.Items)
            {
                if (item.Value == selectedValue)
                {
                    foundListItem = true;
                    item.Selected = true;
                }
                else
                {
                    item.Selected = false;
                }
            }

            if (foundListItem == false)
            {
                ListItem item = new ListItem(selectedText, selectedValue);
                item.Selected = true;

                control.Items.Add(item);
            }
        }

        /// <summary>
        /// Sets the selection if you are using a control for true and a control for false (for example, two RadioButtons).
        /// </summary>
        /// <param name="trueControl"></param>
        /// <param name="falseControl"></param>
        /// <param name="value"></param>
        public static void SetBooleanOnRadioButtonControls(CheckBox trueControl, CheckBox falseControl, bool value)
        {
            if (value == false)
            {
                trueControl.Checked = false;
                falseControl.Checked = true;
            }
            else
            {
                trueControl.Checked = true;
                falseControl.Checked = false;
            }
        }

        /// <summary>
        /// Gets the selection if you are using a control for true and a control for false (for example, two RadioButtons).
        /// </summary>
        /// <param name="trueControl"></param>
        /// <param name="falseControl"></param>
        public static bool GetBooleanFromRadioButtonControls(CheckBox trueControl, CheckBox falseControl)
        {
            if (trueControl.Checked == true)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public static string GetLastDayOfTheMonth()
        {
            DateTime monthFromToday = DateTime.Today.AddMonths(1);

            DateTime firstOfNextMonth = new DateTime(monthFromToday.Year, monthFromToday.Month, 1);

            DateTime lastDayOfThisMonth = firstOfNextMonth.AddDays(-1);

            return lastDayOfThisMonth.ToShortDateString();
        }

        public static Control FindControlById(Control parent, string childId, bool recurse)
        {
            if (parent == null)
                throw new ArgumentNullException("parent", "Argument cannot be null.");
            else if (string.IsNullOrEmpty(childId))
                throw new ArgumentNullException("childId", "Argument cannot be null or empty.");

            Control retValue = null;

            if (parent.HasControls())
            {
                foreach (Control child in parent.Controls)
                {
                    if (child.ID == childId)
                        retValue = child;
                    else if (recurse)
                        retValue = FindControlById(child, childId, recurse);

                    if (retValue != null)
                        break;
                }
            }

            return retValue;
        }

        public static void GetControlsOfType<T>(IList<T> controlList, Control parent, bool recurse) where T : Control
        {
            if (parent == null)
                throw new ArgumentNullException("parent", "Argument cannot be null");

            if (controlList == null)
                controlList = new List<T>();

            if (parent.HasControls())
            {
                foreach (Control child in parent.Controls)
                {
                    Type childType = child.GetType();
                    if (childType == typeof(T))
                        controlList.Add((T)child);

                    if (recurse)
                        GetControlsOfType<T>(controlList, child, recurse);
                }
            }
        }

        public static bool HasBaseType(Type typeToCheck, Type baseType)
        {
            bool retValue = false;

            if (typeToCheck != null && baseType != null)
            {
                if (typeToCheck.BaseType != null)
                {
                    if (typeToCheck.BaseType.Equals(baseType))
                        retValue = true;
                    else
                        retValue = HasBaseType(typeToCheck.BaseType, baseType);
                }
            }

            return retValue;
        }        
    }
}
